@@ -1,10 +1,33 @@
+## ~~~~ VERSION ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+Matrix.Version <- function() {
+    n <- .Call(R_Matrix_version)
+    v <- .mapply(function(n, p, b, class) {
+                     r <- integer(p)
+                     while (p > 0L) {
+                         r[p] <- tmp <- n %% b
+                         n <- (n - tmp) %/% b
+                         p <- p - 1L
+                     }
+                     v <- list(r)
+                     class(v) <- c(class, "numeric_version")
+                     v
+                 },
+                 list(n = n, p = c(3L, 1L, 3L), b = c(256L, 10L, 256L),
+                      class = list("package_version", NULL, NULL)),
+                 NULL)
+    names(v) <- names(n)
+    v
+}
+
+
 ## ~~~~ PACKAGE ENVIRONMENTS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 ## Recording default values of Matrix.* options
 .MatrixEnv <- new.env(parent = emptyenv(), hash = FALSE)
 
 ## Storing settings from 'cholmod_common'
-.chm_common <- new.env(parent = emptyenv())
+.CholmodCommonEnv <- new.env(parent = emptyenv())
 
 
 ## ~~~~ NAMESPACE HOOKS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -23,13 +46,20 @@
                        paste(deparse(expr, width.cutoff, ...),
                              collapse = collapse))
             assign("sequence.default", envir = Mns, inherits = FALSE,
-                   function(nvec, from = 1L, by = 1L, ...)
+                   function(nvec, from = 1L, by = 1L, ...) {
+                       if(length(nvec) == 0L)
+                           return(integer(0L))
+                       else if(length(from) == 0L || length(by) == 0L)
+                           stop(gettextf("'%s' has length 0 but '%s' does not",
+                                         if(length(from) == 0L) "from" else "by", "nvec"),
+                                domain = NA)
                        unlist(.mapply(seq.int,
                                       list(from = as.integer(from),
                                            by = as.integer(by),
                                            length.out = as.integer(nvec)),
                                       NULL),
-                              recursive = FALSE, use.names = FALSE))
+                              recursive = FALSE, use.names = FALSE)
+                   })
             assign("tryInvokeRestart", envir = Mns, inherits = FALSE,
                    function(r, ...)
                        tryCatch(invokeRestart(r, ...),
@@ -73,9 +103,9 @@
     ## >=2 ... persistent error
     ##  NA ... one-time warning
     wSD <- as.integer(Sys.getenv("R_MATRIX_WARN_SQRT_DEFAULT", NA))
-    assign("warnSqrtDefault", wDC, envir = .MatrixEnv)
+    assign("warnSqrtDefault", wSD, envir = .MatrixEnv)
 
-    .Call(CHM_set_common_env, .chm_common)
+    .Call(R_cholmod_common_envini, .CholmodCommonEnv)
     NULL
 }
 
@@ -90,16 +120,18 @@
 ## ~~~~ DEPRECATED ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 ..2dge <- function(from) {
-    .Deprecated(new = ".dense2g", package = "Matrix")
-    .dense2g(from, "d")
+    .Deprecated(new = ".M2gen", package = "Matrix")
+    if(isS4(from))
+        .M2gen(from, "d")
+    else .m2dense(from, "dge")
 }
 .C2nC <- function(from, isTri) {
-    .Deprecated(new = ".sparse2kind", package = "Matrix")
-    .sparse2kind(from, "n")
+    .Deprecated(new = ".M2kind", package = "Matrix")
+    .M2kind(from, "n")
 }
 .T2Cmat <- function(from, isTri) {
-    .Deprecated(new = ".T2CR", package = "Matrix")
-    .T2CR(from, Csparse = TRUE)
+    .Deprecated(new = ".M2C", package = "Matrix")
+    .M2C(from)
 }
 .asmatrix <- function(x) {
     .Deprecated(new = "as(., \"matrix\")", package = "Matrix")
@@ -110,33 +142,37 @@
     .M2sym(from, ...)
 }
 .diag2mat <- function(from) {
-    .Deprecated(new = "as(., \"matrix\")", package = "Matrix")
-    .diag2m(from)
+    .Deprecated(new = ".M2m", package = "Matrix")
+    .M2m(from)
 }
 .diag2sT <- function(from, uplo = "U", kind = ".", drop0 = TRUE) {
     .Deprecated(new = ".diag2sparse", package = "Matrix")
-    .diag2sparse(from, code = `substr<-`(".sT", 1L, 1L, kind),
-                 uplo = uplo, drop0 = drop0)
+    r <- .diag2sparse(from, kind, "s", "T", uplo)
+    if(drop0)
+        r <- .drop0(r)
+    r
 }
 .diag2tT <- function(from, uplo = "U", kind = ".", drop0 = TRUE) {
     .Deprecated(new = ".diag2sparse", package = "Matrix")
-    .diag2sparse(from, code = `substr<-`(".tT", 1L, 1L, kind),
-                 uplo = uplo, drop0 = drop0)
+    to <- .diag2sparse(from, kind, "t", "T", uplo)
+    if(drop0)
+        to <- .drop0(to)
+    to
 }
 .dsy2dsp <- function(from) {
-    .Deprecated(new = "pack", package = "Matrix")
-    pack(from)
+    .Deprecated(new = ".M2packed", package = "Matrix")
+    .M2packed(from)
 }
 .dsy2mat <- function(from, keep.dimnames = TRUE) {
-    .Deprecated(new = ".dense2m", package = "Matrix")
-    to <- .dense2m(from)
-    if (!keep.dimnames)
+    .Deprecated(new = ".M2m", package = "Matrix")
+    to <- .M2m(from)
+    if(!keep.dimnames)
         dimnames(to) <- NULL
     to
 }
 .dxC2mat <- function(from, chkUdiag) {
-    .Deprecated(new = ".sparse2m", package = "Matrix")
-    .sparse2m(from)
+    .Deprecated(new = ".M2m", package = "Matrix")
+    .M2m(from)
 }
 .m2dgC <- function(from) {
     .Deprecated(new = ".m2sparse", package = "Matrix")
@@ -149,32 +185,126 @@
 .m2ngC <- function(from) {
     .Deprecated(new = ".m2sparse", package = "Matrix")
     if(anyNA(from))
-        stop("attempt to coerce matrix with NA to ngCMatrix")
+        stop(gettextf("attempt to coerce matrix with NA to %s", "ngCMatrix"),
+             domain = NA)
     .m2sparse(from, "ngC")
 }
 .m2ngCn <- function(from, na.is.not.0 = FALSE) {
     .Deprecated(new = ".m2sparse", package = "Matrix")
     if(!na.is.not.0 && anyNA(from))
-        stop("attempt to coerce matrix with NA to ngCMatrix")
+        stop(gettextf("attempt to coerce matrix with NA to %s", "ngCMatrix"),
+             domain = NA)
     .m2sparse(from, "ngC")
 }
 .m2ngTn <- function(from, na.is.not.0 = FALSE) {
     .Deprecated(new = ".m2sparse", package = "Matrix")
     if(!na.is.not.0 && anyNA(from))
-        stop("attempt to coerce matrix with NA to ngCMatrix")
+        stop(gettextf("attempt to coerce matrix with NA to %s", "ngTMatrix"),
+             domain = NA)
     .m2sparse(from, "ngT")
 }
 .n2dgT <- function(from) {
-    .Deprecated(new = ".sparse2kind", package = "Matrix")
-    .sparse2kind(from, "d")
+    .Deprecated(new = ".M2kind", package = "Matrix")
+    .M2kind(from, "d")
 }
 .nC2d <- function(from) {
-    .Deprecated(new = ".sparse2kind", package = "Matrix")
-    .sparse2kind(from, "d")
+    .Deprecated(new = ".M2kind", package = "Matrix")
+    .M2kind(from, "d")
 }
 .nC2l <- function(from) {
-    .Deprecated(new = ".sparse2kind", package = "Matrix")
-    .sparse2kind(from, "l")
+    .Deprecated(new = ".M2kind", package = "Matrix")
+    .M2kind(from, "l")
+}
+
+.dense2m <- .sparse2m <- function(from) {
+    if(FALSE) {
+    .Deprecated(new = ".M2m", package = "Matrix")
+    }
+    .M2m(from)
+}
+
+.dense2v <- .sparse2v <- function(from) {
+    if(FALSE) {
+    .Deprecated(new = ".M2v", package = "Matrix")
+    }
+    .M2v(from)
+}
+
+.dense2kind <- function(from, kind) {
+    if(FALSE) {
+    .Deprecated(new = ".M2kind", package = "Matrix")
+    }
+    .M2kind(from, kind)
+}
+
+.sparse2kind <- function(from, kind, drop0 = FALSE) {
+    if(FALSE) {
+    .Deprecated(new = ".M2kind", package = "Matrix")
+    }
+    .M2kind(if(drop0) .drop0(from) else from, kind)
+}
+
+.dense2g <- .sparse2g <- function(from, kind = ".") {
+    if(FALSE) {
+    .Deprecated(new = ".M2gen", package = "Matrix")
+    }
+    .M2gen(from, kind)
+}
+
+.CR2RC <- function(from) {
+    if(.M.repr(from) != "C") {
+        if(FALSE) {
+        .Deprecated(new = ".M2C", package = "Matrix")
+        }
+        .M2C(from)
+    } else {
+        if(FALSE) {
+        .Deprecated(new = ".M2R", package = "Matrix")
+        }
+        .M2R(from)
+    }
+}
+
+.CR2T <- function(from) {
+    if(FALSE) {
+    .Deprecated(new = ".M2T", package = "Matrix")
+    }
+    .M2T(from)
+}
+
+.T2CR <- function(from, Csparse = TRUE) {
+    if(Csparse) {
+        if(FALSE) {
+        .Deprecated(new = ".M2C", package = "Matrix")
+        }
+        .M2C(from)
+    } else {
+        if(FALSE) {
+        .Deprecated(new = ".M2R", package = "Matrix")
+        }
+        .M2R(from)
+    }
+}
+
+.tCR2RC <- function(from) {
+    if(FALSE) {
+    .Deprecated(new = ".tCRT", package = "Matrix")
+    }
+    .tCRT(from)
+}
+
+uniqTsparse <- function(x, class.x = class(x)) {
+    if(FALSE) {
+    .Deprecated(new = "asUniqueT", package = "Matrix")
+    }
+    asUniqueT(x, isT = extends(class.x, "TsparseMatrix"))
+}
+
+.SuiteSparse_version <- function() {
+    if(FALSE) {
+    .Deprecated(new = "Matrix.Version", package = "Matrix")
+    }
+    Matrix.Version()[["suitesparse"]]
 }
 
 ## Utility for Matrix.DeprecatedCoerce(); see below
@@ -473,11 +603,3 @@ cBind <- function(..., deparse.level = 1)
     .Defunct(new = "cbind", package = "Matrix")
 rBind <- function(..., deparse.level = 1)
     .Defunct(msg = "rbind", package = "Matrix")
-
-
-## ~~~~ "MISCELLANEOUS" ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-.SuiteSparse_version <- function() {
-    v <- .Call(get_SuiteSparse_version)
-    package_version(list(major = v[1L], minor = paste(v[2:3], collapse = ".")))
-}
