@@ -113,8 +113,8 @@ dsy <- lsy+1
 D3 <- Diagonal(x=4:2); L7 <- Diagonal(7) > 0
 validObject(xpp <- pack(round(xpx,2)))
 lsp <- xpp > 0
-(dsyU <- .diag2dense(D3, "dsy"))
- lsyU <- .diag2dense(Diagonal(5) > 0, "lsy")
+(dsyU <- .diag2dense(D3, ".", "s"))
+ lsyU <- .diag2dense(Diagonal(5) > 0, ".", "s")
 str(lsyU)
 stopifnot({
     isValid(dsyU,               "dsyMatrix") && dsyU@uplo == "U"
@@ -208,7 +208,9 @@ local({
                                grep("bogomips", readLines("/proc/cpuinfo"),
                                     ignore.case=TRUE, # e.g. ARM : "BogoMIPS"
                                     value=TRUE)[[1]])))
-        if(is.numeric(mips) && all(mips) > 0)
+        print(mips)
+        if(is.numeric(mips) && all(mips) > 0 && doExtras)
+                                        # doExtras: valgrind (2023-07-26) gave large 'st[1]'
         stopifnot(st[1] < 1000/mips)# ensure there was no gross inefficiency
     }
 })
@@ -238,7 +240,7 @@ stopifnot(validObject(lm1), validObject(lm2),
           identical(dsc, dsc * as(lm1, "dMatrix")))
 
 crossprod(lm1) # lm1: "lsC*"
-cnm1 <- crossprod(nm1)
+cnm1 <- crossprod(nm1, boolArith = FALSE)
 stopifnot(is(cnm1, "symmetricMatrix"), ## whereas the %*% is not:
 	  Q.eq(cnm1, nm1 %*% nm1))
 dn1 <- as(nm1, "denseMatrix")
@@ -284,7 +286,7 @@ stopifnot(sv == colSums(kC), is.na(as.vector(ddv)) == na.ddv,
 ## Subclasses (!)
 setClass("m.spV", contains = "dsparseVector")
 (m.ddv <- as(ddv, "m.spV"))
-stopifnot(all.equal(m.ddv, ddv))# failed
+stopifnot(all.equal(m.ddv, ddv, check.class = FALSE))# failed
 setClass("m.dgC", contains = "dgCMatrix")
 (m.mC <- as(mC, "m.dgC"))
 stopifnot(all(m.mC == mC))
@@ -370,7 +372,7 @@ stopifnot(exprs = {
 
 options(op)
 
-if(!doExtras && !interactive()) q("no") ## (saving testing time)
+if(doExtras || interactive()) { # save testing time
 
 ### Systematically look at all "Ops" group generics for "all" Matrix classes
 ### -------------- Main issue: Detect infinite recursion problems
@@ -455,6 +457,8 @@ for(gr in getGroupMembers("Ops")) {
 }
 if(length(warnings())) print(summary(warnings()))
 showProc.time()
+options(op) # reset 'warn'
+} # doExtras
 
 ###---- Now checking 0-length / 0-dim cases  <==> to R >= 3.4.0 !
 
@@ -502,7 +506,7 @@ stopifnot(identical(Matrix(3,1,1) > NULL, T[0]))
 stopifnot(identical(Matrix(3,1,1) & NULL, T[0]))
 ## in R >= 3.4.0: logical(0) # with *no* warning and that's correct!
 
-options(op)# reset 'warn'
+if(doExtras || interactive()) { # save testing time
 mStop <- function(...) stop(..., call. = FALSE)
 ##
 cat("Checking the Math (+ Math2) group generics for a set of arguments:\n",
@@ -524,9 +528,9 @@ for(f in c(mM, mM2)) {
     r <- fn(m <- if(is.m) as.mat(M) else as.vector(M))
     stopifnot(identical(dim(R), dim(r)))
     if(givesVec || !is.m) {
-        assert.EQ(R, r)
+        assert.EQ(R, r, check.class = FALSE)
     } else { ## (almost always:) matrix result
-        assert.EQ.mat(R, r)
+        assert.EQ.mat(R, r, check.class = FALSE)
 	## check preservation of properties, notably super class
 	if(prod(dim(M)) > 1 && is(M, "diagonalMatrix"  ) && isDiagonal(R) && !is(R, "diagonalMatrix"  )) doStop()
 	if(prod(dim(M)) > 1 && is(M, "triangularMatrix") && (iT <- isTriangular(R)) && attr(iT, "kind") == M@uplo &&
@@ -557,6 +561,7 @@ for(f in getGroupMembers("Summary")) {
   cat("\n")
   if(length(warnings())) print(summary(warnings()))
 }
+} # doExtras
 
 ## <Math>(x) behaved incorrectly in Matrix <= 1.4-1
 ## for unit diagonal 'x' when f(0) == 0 and f(1) != 1
@@ -570,4 +575,35 @@ sinu <- `dimnames<-`(sin(diag(2L)), Dn)
 for(u in list(udi, utC, utr))
     stopifnot(identical(as(sin(u), "matrix"), sinu))
 
+## Originally in ../man/all-methods.Rd :
+M <- Matrix(1:12 +0, 3,4)
+all(M >= 1) # TRUE
+any(M < 0 ) # FALSE
+MN <- M; MN[2,3] <- NA; MN
+all(MN >= 0) # NA
+any(MN <  0) # NA
+any(MN <  0, na.rm = TRUE) # -> FALSE
+sM <- as(MN, "sparseMatrix")
+stopifnot(all(M >= 1), !any(M < 0),
+          all.equal((sM >= 1), as(MN >= 1, "sparseMatrix")),
+          ## MN:
+          any(MN < 2), !all(MN < 5),
+          is.na(all(MN >= 0)), is.na(any(MN < 0)),
+          all(MN >= 0, na.rm=TRUE), !any(MN < 0, na.rm=TRUE),
+          ## same for sM :
+          any(sM < 2), !all(sM < 5),
+          is.na(all(sM >= 0)), is.na(any(sM < 0)),
+          all(sM >= 0, na.rm=TRUE), !any(sM < 0, na.rm=TRUE)
+         )
+
+## prod(<symmetricMatrix>) does not perform multiplies in row/column order :
+x4 <- new("dspMatrix", Dim = c(4L, 4L),
+          x = c(171, 53, 79, 205, 100, 285, 98, 15, 99, 84))
+p4 <- prod(x4)
+p4. <- prod(as(x4, "generalMatrix"))
+p4.. <- prod(as(x4, "matrix"))
+stopifnot(all.equal(p4, p4.), identical(p4., p4..))
+all.equal(p4, p4., tolerance = 0)
+.Machine[["sizeof.longdouble"]]
+
 cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''
